# Portions (c) 2014, Alexander Klimenko <alex@erix.ru>
# All rights reserved.
#
# Copyright (c) 2011, SmartFile <btimby@smartfile.com>
# All rights reserved.
#
# This file is part of DjangoDav.
#
# DjangoDav is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DjangoDav is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with DjangoDav.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import django
from django.conf import settings


DEFAULT_SETTINGS = dict(
    INSTALLED_APPS = (
        'djangodav',
        'django.contrib.auth',
        'django.contrib.contenttypes',
        # 'djangodav.tests',
    ),
    DATABASES = dict(
        default = dict(
            ENGINE = 'django.db.backends.sqlite3'
        )
    ),
    ROOT_URLCONF = 'djangodav.tests.urls',
    MIDDLEWARE_CLASSES = ()
)


def runtests(*test_args):
    if not settings.configured:
        settings.configure(**DEFAULT_SETTINGS)
    if hasattr(django, 'setup'):
        django.setup()
    if not test_args:
        test_args = ['djangodav']

    parent = os.path.dirname(os.path.abspath(__file__))
    sys.path.insert(0, parent)
    try:
        from django.test.runner import DiscoverRunner
        runner_class = DiscoverRunner
    except ImportError:
        from django.test.simple import DjangoTestSuiteRunner
        runner_class = DjangoTestSuiteRunner
    failures = runner_class(varbosity=1, interactive=True, failfast=False).run_tests(test_args)
    sys.exit(failures)


if __name__ == '__main__':
    runtests()



# Portions (c) 2014, Alexander Klimenko <alex@erix.ru>
# All rights reserved.
#
# Copyright (c) 2011, SmartFile <btimby@smartfile.com>
# All rights reserved.
#
# This file is part of DjangoDav.
#
# DjangoDav is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DjangoDav is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with DjangoDav.  If not, see <http://www.gnu.org/licenses/>.
from datetime import datetime
from django.utils.timezone import utc
from djangodav.base.resources import BaseDavResource
from mock import Mock, MagicMock


class MockResource(MagicMock, BaseDavResource):
    exists = True
    get_created = Mock(return_value=datetime(1983, 12, 24, 6, tzinfo=utc))
    get_modified = Mock(return_value=datetime(2014, 12, 24, 6, tzinfo=utc))
    getcontentlength = 0

    def __init__(self, path, *args, **kwargs):
        super(MockResource, self).__init__(spec=BaseDavResource, *args, **kwargs)
        BaseDavResource.__init__(self, path)


class MockObject(MockResource):
    getetag = "0" * 40
    is_object = True
    is_collection = False
    getcontentlength = 42


class MockCollection(MockResource):
    is_object = False
    is_collection = True


class MissingMockResource(MockResource):
    exists = False


class MissingMockObject(MissingMockResource):
    is_object = True
    is_collection = False
    getcontentlength = 42


class MissingMockCollection(MissingMockResource):
    is_object = False
    is_collection = True



# Portions (c) 2014, Alexander Klimenko <alex@erix.ru>
# All rights reserved.
#
# Copyright (c) 2011, SmartFile <btimby@smartfile.com>
# All rights reserved.
#
# This file is part of DjangoDav.
#
# DjangoDav is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DjangoDav is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with DjangoDav.  If not, see <http://www.gnu.org/licenses/>.
from djangodav.acls import FullAcl
from djangodav.locks import DummyLock

from djangodav.views import DavView

from django.conf.urls import patterns

from samples.fs.resources import TempDirWebDavResource
from samples.db.resources import MyDBDavResource
from samples.auth.views.rest import RestAuthDavView
from samples.auth.views.tasty import TastyAuthDavView


urlpatterns = patterns('',
    # Mirroring tmp folder
    (r'^fs(?P<path>.*)$', DavView.as_view(resource_class=TempDirWebDavResource, lock_class=DummyLock, acl_class=FullAcl)),
    # Db file keeper
    (r'^db(?P<path>.*)$', DavView.as_view(resource_class=MyDBDavResource, lock_class=DummyLock, acl_class=FullAcl)),

    # REST framework auth
    (r'^auth/rest(?P<path>.*)$', RestAuthDavView.as_view(resource_class=TempDirWebDavResource, lock_class=DummyLock, acl_class=FullAcl)),
    # Tastypie auth
    (r'^auth/tasty(?P<path>.*)$', TastyAuthDavView.as_view(resource_class=TempDirWebDavResource, lock_class=DummyLock, acl_class=FullAcl)),
)



# Portions (c) 2014, Alexander Klimenko <alex@erix.ru>
# All rights reserved.
#
# Copyright (c) 2011, SmartFile <btimby@smartfile.com>
# All rights reserved.
#
# This file is part of DjangoDav.
#
# DjangoDav is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DjangoDav is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with DjangoDav.  If not, see <http://www.gnu.org/licenses/>.
from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from djangodav.responses import HttpResponseUnAuthorized
from rest_framework.exceptions import APIException


class RequestWrapper(object):
    """ simulates django-rest-api request wrapper """
    def __init__(self, request):
        self._request = request
    def __getattr__(self, attr):
        return getattr(self._request, attr)
    
    
class RestAuthViewMixIn(object):
    authentications = NotImplemented

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        if request.method.lower() != 'options':
            user_auth_tuple = None
            for auth in self.authentications:
                try:
                    user_auth_tuple = auth.authenticate(RequestWrapper(request))
                except APIException as e:
                    return HttpResponse(e.detail, status=e.status_code)
                else:
                    if user_auth_tuple is None:
                        continue # try next authenticator 
                    else:
                        break    # we got auth, so stop trying

            if not user_auth_tuple is None:
                user, auth = user_auth_tuple
            else:
                resp = HttpResponseUnAuthorized("Not Authorised")
                resp['WWW-Authenticate'] = self.authentications[0].authenticate_header(request)
                return resp

            request.user = user
            request.auth = auth
        return super(RestAuthViewMixIn, self).dispatch(request, *args, **kwargs)



# Portions (c) 2014, Alexander Klimenko <alex@erix.ru>
# All rights reserved.
#
# Copyright (c) 2011, SmartFile <btimby@smartfile.com>
# All rights reserved.
#
# This file is part of DjangoDav.
#
# DjangoDav is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DjangoDav is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with DjangoDav.  If not, see <http://www.gnu.org/licenses/>.
from django.test import TestCase
from djangodav.fs.resources import BaseFSDavResource
from mock import patch


class TestFSDavResource(TestCase):
    class FSDavResource(BaseFSDavResource):
        base_url = 'http://testserver/base/'
        root = '/some/folder/'

    def setUp(self):
        self.resource = self.FSDavResource("/path/to/name")

    @patch('djangodav.fs.resources.os.path.isdir')
    def test_is_collection(self, isdir):
        isdir.return_value = True
        self.assertTrue(self.resource.is_collection)
        isdir.assert_called_with('/some/folder/path/to/name')

    @patch('djangodav.fs.resources.os.path.isfile')
    def test_isfile(self, isfile):
        isfile.return_value = True
        self.assertTrue(self.resource.is_object)
        isfile.assert_called_with('/some/folder/path/to/name')

    @patch('djangodav.fs.resources.os.path.exists')
    def test_isfile(self, exists):
        exists.return_value = True
        self.assertTrue(self.resource.exists)
        exists.assert_called_with('/some/folder/path/to/name')

    @patch('djangodav.fs.resources.os.path.getsize')
    def test_get_size(self, getsize):
        getsize.return_value = 42
        self.assertEquals(self.resource.getcontentlength, 42)
        getsize.assert_called_with('/some/folder/path/to/name')

    def test_get_abs_path(self):
        self.assertEquals(self.resource.get_abs_path(), '/some/folder/path/to/name')

    @patch('djangodav.fs.resources.os.listdir')
    def test_get_children(self, listdir):
        listdir.return_value=['child1', 'child2']
        children = list(self.resource.get_children())
        self.assertEqual(children[0].path, ['path', 'to', 'name', 'child1'])
        self.assertEqual(children[1].path, ['path', 'to', 'name', 'child2'])
        listdir.assert_called_with('/some/folder/path/to/name')



# Portions (c) 2014, Alexander Klimenko <alex@erix.ru>
# All rights reserved.
#
# Copyright (c) 2011, SmartFile <btimby@smartfile.com>
# All rights reserved.
#
# This file is part of DjangoDav.
#
# DjangoDav is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DjangoDav is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with DjangoDav.  If not, see <http://www.gnu.org/licenses/>.


class BaseLock(object):
    def __init__(self, resource):
        self.resource = resource

    def get(self):
        """Gets all active locks for the requested resource. Returns a list of locks."""
        raise NotImplementedError()

    def acquire(self, lockscope, locktype, depth, timeout, owner):
        """Creates a new lock for the given resource."""
        raise NotImplementedError()

    def release(self, token):
        """Releases the lock referenced by the given lock id."""
        raise NotImplementedError()

    def del_locks(self):
        """Releases all locks for the given resource."""
        raise NotImplementedError()



# Portions (c) 2014, Alexander Klimenko <alex@erix.ru>
# All rights reserved.
#
# Copyright (c) 2011, SmartFile <btimby@smartfile.com>
# All rights reserved.
#
# This file is part of DjangoDav.
#
# DjangoDav is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DjangoDav is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with DjangoDav.  If not, see <http://www.gnu.org/licenses/>.
from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from djangodav.responses import HttpResponseUnAuthorized


class TastypieAuthViewMixIn(object):
    authentication = NotImplemented

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):

        if request.method.lower() != 'options':
            auth_result = self.authentication.is_authenticated(request)

            if isinstance(auth_result, HttpResponse):
                return auth_result

            if not auth_result is True:
                return HttpResponseUnAuthorized()

        return super(TastypieAuthViewMixIn, self).dispatch(request, *args, **kwargs)



# Portions (c) 2014, Alexander Klimenko <alex@erix.ru>
# All rights reserved.
#
# Copyright (c) 2011, SmartFile <btimby@smartfile.com>
# All rights reserved.
#
# This file is part of DjangoDav.
#
# DjangoDav is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DjangoDav is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with DjangoDav.  If not, see <http://www.gnu.org/licenses/>.
from django.test import TestCase
from djangodav.base.resources import BaseDavResource
from djangodav.base.tests.resources import MockCollection, MockObject, MissingMockCollection
from mock import patch, Mock


class TestBaseDavResource(TestCase):

    def setUp(self):
        self.resource = BaseDavResource("/path/to/name")

    def test_path(self):
        self.assertEqual(self.resource.path, ['path', 'to', 'name'])

    @patch('djangodav.base.resources.BaseDavResource.is_collection', True)
    def test_get_path_collection(self):
        self.assertEqual(self.resource.get_path(), '/path/to/name/')

    @patch('djangodav.base.resources.BaseDavResource.is_collection', False)
    def test_get_path_object(self):
        self.assertEqual(self.resource.get_path(), '/path/to/name')

    @patch('djangodav.base.resources.BaseDavResource.get_children', Mock(return_value=[]))
    def test_get_descendants(self):
        self.assertEqual(list(self.resource.get_descendants(depth=1, include_self=True)), [self.resource])

    def test_get_parent_path(self):
        self.assertEqual(self.resource.get_parent_path(), '/path/to/')

    def test_displayname(self):
        self.assertEqual(self.resource.displayname, 'name')

    def test_move_collection(self):
        child = MockObject('/path/to/src/child', move=Mock())
        src = MockCollection('/path/to/src/', get_children=Mock(return_value=[child]), delete=Mock())
        dst = MissingMockCollection('/path/to/dst/', create_collection=Mock())

        src.move(dst)

        src.delete.assert_called_with()
        dst.create_collection.assert_called_with()
        self.assertEqual(child.move.call_args[0][0].path, ['path', 'to', 'dst', 'child'])

    def test_move_collection_collision(self):
        child = MockObject('/path/to/src/child', move=Mock())
        src = MockCollection('/path/to/src/', get_children=Mock(return_value=[child]), delete=Mock())
        dst = MockCollection('/path/to/dst/', create_collection=Mock())

        src.move(dst)

        src.delete.assert_called_with()
        self.assertEqual(dst.create_collection.call_count, 0)
        self.assertEqual(child.move.call_args[0][0].path, ['path', 'to', 'dst', 'child'])

    def test_copy_collection(self):
        child = MockObject('/path/to/src/child', copy=Mock())
        src = MockCollection('/path/to/src/', get_children=Mock(return_value=[child]), delete=Mock())
        dst = MissingMockCollection('/path/to/dst/', create_collection=Mock())

        src.copy(dst)

        dst.create_collection.assert_called_with()
        self.assertEqual(child.copy.call_args[0][0].path, ['path', 'to', 'dst', 'child'])

    def test_copy_collection_collision(self):
        child = MockObject('/path/to/src/child', copy=Mock())
        src = MockCollection('/path/to/src/', get_children=Mock(return_value=[child]), delete=Mock())
        dst = MockCollection('/path/to/dst/', create_collection=Mock())

        src.copy_collection(dst)

        self.assertEqual(dst.create_collection.call_count, 0)
        self.assertEqual(child.copy.call_args[0][0].path, ['path', 'to', 'dst', 'child'])

    def test_copy_collection_depth_0(self):
        child = MockObject('/path/to/src/child', copy=Mock())
        src = MockCollection('/path/to/src/', get_children=Mock(return_value=[child]), delete=Mock())
        dst = MissingMockCollection('/path/to/dst/', create_collection=Mock())

        src.copy(dst, 0)

        dst.create_collection.assert_called_with()
        self.assertEqual(child.copy.call_count, 0)



from os.path import dirname
from base64 import b64encode

from django.test import TestCase
from django.test.client import RequestFactory

from djangodav.views import DavView
from djangodav.fs.resources import DummyReadFSDavResource
from djangodav.auth.rest import RestAuthViewMixIn

from rest_framework.authentication import SessionAuthentication as RestSessionAuthentication, \
                                          BasicAuthentication as RestBasicAuthentication
from djangodav.responses import HttpResponseUnAuthorized
from django.contrib.auth import get_user_model
from djangodav.locks import DummyLock
from djangodav.acls import ReadOnlyAcl
from django.http.response import HttpResponse

class TestFSResource(DummyReadFSDavResource):
    """ just a test resource """
    root = dirname(__file__)

class TestDAVView(RestAuthViewMixIn, DavView):
    """ dummy view """
    resource_class= TestFSResource 
    acl_class = ReadOnlyAcl
    lock_class = DummyLock 
            
class RestAuthTest(TestCase):
    """ test authentication through Django Rest Framework. This will
        only work when RestFramework is actually installed.          """
    
    def setUp(self):
        self.user = get_user_model()(username='root', is_active=True)
        self.user.set_password('test')
        self.user.save()
    
    def assertIsAuthorized(self, response):
        self.assertIsInstance(response, HttpResponse)
        self.assertNotIsInstance(response, HttpResponseUnAuthorized)
        
    def assertIsNotAuthorized(self, response):
        self.assertIsInstance(response, HttpResponseUnAuthorized)
      
    def assertHasAuthenticateHeader(self, response):
        self.assertEqual(response['WWW-Authenticate'], 'Basic realm="api"')
      
    def test_auth_session(self):
        """ test whether we can authenticate through Django session """        
        
        class RestAuthDavView(TestDAVView):
            authentications = (RestSessionAuthentication(),)
        v = RestAuthDavView.as_view()
            
        # get with no authentication yields HttpResponseUnAuthorized
        request = RequestFactory().get('/')
        response = v(request, '/')
        self.assertIsNotAuthorized(response)
    
        # get with authentication goes through
        request = RequestFactory().get('/')
        # in the regular case, session handling would be done in
        # middleware. RestSessionAuthentication only checks for
        # request.user, so we just fake that
        request.user = self.user
        response = v(request, '/')
        self.assertIsAuthorized(response)
    
    def test_auth_basic(self):
        """ test whether we can authenticate through Basic auth """

        class RestAuthDavView(TestDAVView):
            authentications = (RestBasicAuthentication(),)
        v = RestAuthDavView.as_view()
        
        # get with no authentication yields HttpResponseUnAuthorized
        request = RequestFactory().get('/')
        response = v(request, '/')
        self.assertIsNotAuthorized(response)
        self.assertHasAuthenticateHeader(response)

        # get with session authentication does not get through
        request = RequestFactory().get('/')
        request.user = self.user
        response = v(request, '/')
        self.assertIsNotAuthorized(response)

        # get with basic authentication goes through
        request = RequestFactory().get('/', **{'HTTP_AUTHORIZATION': 'Basic %s' % b64encode('root:test')})
        response = v(request, '/')
        self.assertIsAuthorized(response)
    
    def test_auth_multiple(self):
        """ test whether we can authenticate through either Session or Basic auth """
        
        class RestAuthDavView(TestDAVView):
            authentications = (RestBasicAuthentication(), RestSessionAuthentication(),)
        v = RestAuthDavView.as_view()
        
        # get with no authentication yields HttpResponseUnAuthorized
        request = RequestFactory().get('/')
        response = v(request, '/')
        self.assertIsNotAuthorized(response)
        self.assertHasAuthenticateHeader(response)
        
        # get with session authentication goes through
        request = RequestFactory().get('/')
        request.user = self.user
        response = v(request, '/')
        self.assertIsAuthorized(response)    
    
        # get with basic authentication goes through
        request = RequestFactory().get('/', **{'HTTP_AUTHORIZATION': 'Basic %s' % b64encode('root:test')})
        response = v(request, '/')
        self.assertIsAuthorized(response)



#!/usr/bin/env python
#
# Portions (c) 2014, Alexander Klimenko <alex@erix.ru>
# All rights reserved.
#
# Copyright (c) 2011, SmartFile <btimby@smartfile.com>
# All rights reserved.
#
# This file is part of DjangoDav.
#
# DjangoDav is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DjangoDav is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with DjangoDav.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, find_packages


setup(
    name='DjangoDav',
    version=__import__('djangodav').__version__,
    description=('A WebDAV server for Django.'),
    long_description = open('README.rst').read(),
    author='Alexander Klimenko',
    author_email='alex@erix.ru',
    url='https://github.com/meteozond/djangodav',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 3.4",
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    install_requires=["lxml", "Django>=1.3.0"],
    tests_require=["Django>=1.3.0", "mock==1.0.1"],
    include_package_data=True,
    zip_safe=False,
    test_suite='runtests.runtests'
)
